@@ -189,7 +189,8 @@
 #define PCAP_AVAILABLE_1_7	PCAP_AVAILABLE_MACOS(10.12)
 #define PCAP_AVAILABLE_1_8	PCAP_AVAILABLE_MACOS(10.13) /* only Windows adds routines to the API; XXX - what version first had it? */
 #define PCAP_AVAILABLE_1_9	PCAP_AVAILABLE_MACOS(10.13)
-#define PCAP_AVAILABLE_1_10	/* not released yet, so not in macOS yet */
+#define PCAP_AVAILABLE_1_10	/* not in macOS yet */
+#define PCAP_AVAILABLE_1_11	/* not released yet, so not in macOS yet */ 
 #else /* __APPLE__ */
 #define PCAP_AVAILABLE_0_4
 #define PCAP_AVAILABLE_0_5
@@ -208,6 +209,7 @@
 #define PCAP_AVAILABLE_1_8
 #define PCAP_AVAILABLE_1_9
 #define PCAP_AVAILABLE_1_10
+#define PCAP_AVAILABLE_1_11
 #endif /* __APPLE__ */
 
 /*
